;;; This is a generated file, do not edit it!
(setq vm-version "vm-8.0.12-devo-585")
(setq vm-version-info '(
  (revdate "2008-08-13 00:18:56 +0200")
  (revno 585)
  (revid "hack@robf.de-20080812221856-lsbvda4jidjgd7a4")
  (branch_nick "vm-8.0.x-xemacs")
  (author "Robert Widhopf-Fenk <hack@robf.de>")
))
